
class WidgetFinder:
    """Finds widgets.
    """
    def __init__(self, levels=1):
        """Create a widgetfinder.

        levels -- the amount of priority levels (default is a single level)
        """
        # set nodewidgets dict
        self.nodewidgets = {}
        
        # set list of dicts for levels
        levellist = []
        for i in range(levels):
            levellist.append({})

        self.levellist = levellist
        
    def find(self, node, REQUEST=None):
        """Try to find widget in list of widget levels.

        node -- node to find widget for
        REQUEST -- needs to be passed to make Zope authentication play nice
        Result -- the first widget found, or None if not found
        
        Levels with a higher index in the levellist have higher priority.
        """
        # first check if special widget exists for this node
        widget = self.nodewidgets.get(node.absolute_url(), None)
        if widget is not None:
            return widget
    
        levellist = self.levellist[:]
        levellist.reverse()
        for level in levellist:
            widget = self._findInLevel(node, level, REQUEST)
            if widget != None:
                return widget        
        return None
    
    def _findInLevel(self, node, level, REQUEST):
        """Try to find widget in particular priority level.

        node -- node to find widget for
        level -- the level to look in
        REQUEST -- Zope request object polluting this interface
                   necessary to pass for Zope authentication reasons,
                   if _isNodeSupported() calls DTML method.
        Result -- the first widget found, or None if not found
        """
        widgetlist = level.get(node.getNodeName(), [])
        for widget in widgetlist:
            # return the first widget that says it's the right one
            if widget._isNodeSupported(node, REQUEST):
                return widget
        return None # didn't find it
    
    def addWidget(self, widget, level=0):
        """Add a widget to the widgetfinder.

        widget -- the widget to add
        level -- the level to add the widget at (default, lowest priority)
        """
        levellist = self.levellist
        
        widgetlist = levellist[level].get(widget.nodename, [])
        widgetlist.append(widget)
        levellist[level][widget.nodename] = widgetlist

        self.levellist = levellist

    def addNodeWidget(self, widget, node):
        """Add a widget for a specific node to the widgetfinder.

        widget -- the widget to add
        node -- the node to add the widget for
        """
        nodewidgets = self.nodewidgets
        
        nodewidgets[node.absolute_url()] = widget

        self.nodewidgets = nodewidgets
        
    def clearLevel(self, level=0):
        """Clear an entire priority level of widgets.

        level -- the priority level to clear.
        """
        levellist = self.levellist

        levellist[level] = {}

        self.levellist = levellist

    def clearNodeWidgets(self):
        """Clear node widgets dict.
        """
        self.nodewidgets = {}
